import { MockedProvider } from '@apollo/client/testing'
import { MockedResponse } from '@apollo/client/testing/core'
import { fireEvent, render, waitFor } from '@testing-library/react-native'
import expect from 'expect'
import React from 'react'

import { HoliError } from '@holi/core/errors/classes/HoliError'
import { createSpaceList } from '@holi/core/screens/spaces/__tests__/testData'
import SpacesGroupSlider from '@holi/core/screens/spaces/discover/components/SpacesGroupSlider'
import {
  DiscoverSpacesData,
  DiscoverSpacesGroup,
  DiscoverSpacesVariables,
  discoverSpacesQuery,
} from '@holi/core/screens/spaces/discover/queries'
import { Space } from '@holi/core/screens/spaces/types'
import { Skill, Topic } from '@holi/core/screens/userprofile/types'
import { HoliToastType } from '@holi/ui/components/molecules/HoliToastProvider'

const topic: Topic = { id: 'topic1', title: 'Topic 1', slug: 'topic-1', isDiscoverSpacesDefault: false }

const skill: Skill = { id: 'skill1', title: 'Skill 1', slug: 'skill-1' }

// prevent logging about incomplete apollo mocks,
// because we didn't flesh out the response data
global.console.error = jest.fn()
global.console.warn = jest.fn()

const discoverSpacesErrorMock = ({
  topicIds,
  skillIds,
}: {
  topicIds?: string[]
  skillIds?: string[]
}): MockedResponse<DiscoverSpacesData, DiscoverSpacesVariables> => ({
  request: {
    query: discoverSpacesQuery,
    variables: {
      topicIds,
      skillIds,
      limit: 5,
      offset: 0,
    },
  },
  error: new HoliError('Test error', 'test-error'),
})

const discoverSpacesDataMock = ({
  topicIds,
  skillIds,
}: {
  topicIds?: string[]
  skillIds?: string[]
}): MockedResponse<DiscoverSpacesData, DiscoverSpacesVariables> => {
  const spaces: Space[] = createSpaceList(5)
  const discoverSpaces: DiscoverSpacesGroup[] = [
    ...(topicIds?.map((topicId) => ({
      topicId,
      spaces: {
        data: spaces,
        totalResults: 100,
      },
    })) || []),
    ...(skillIds?.map((skillId) => ({
      skillId,
      spaces: {
        data: spaces,
        totalResults: 5,
      },
    })) || []),
  ]
  return {
    request: {
      query: discoverSpacesQuery,
      variables: {
        topicIds,
        skillIds,
        limit: 5,
        offset: 0,
      },
    },
    result: {
      data: {
        discoverSpaces,
      },
    },
  }
}

describe('SpacesGroupSlider', () => {
  describe('with topic', () => {
    const renderTopic = (mocks: MockedResponse[]) =>
      render(
        <MockedProvider mocks={mocks} addTypename={false}>
          <SpacesGroupSlider topic={topic} pageSize={5} />
        </MockedProvider>
      )

    it('displays errors', async () => {
      const { getByText, getAllByTestId } = renderTopic([discoverSpacesErrorMock({ topicIds: ['topic1'] })])

      expect(getByText('Topic 1')).toBeDefined()

      expect(getAllByTestId('space-link-loading')).toHaveLength(5)

      await waitFor(() =>
        expect(global.mockOpenToast).toHaveBeenCalledWith('spaces.spaceListing.error', HoliToastType.ERROR, {
          testID: 'test-error',
        })
      )
    })

    it('renders correctly with see more', async () => {
      const { getByText, queryAllByTestId, getAllByText, getAllByRole, getByTestId } = renderTopic([
        discoverSpacesDataMock({ topicIds: ['topic1'] }),
      ])

      expect(getByText('Topic 1')).toBeDefined()

      expect(queryAllByTestId('space-link-loading')).toHaveLength(5)
      expect(queryAllByTestId('spaces-group-slider-see-more')).toHaveLength(0)

      await waitFor(async () => {
        expect(queryAllByTestId('space-link-loading')).toHaveLength(0)
      })

      expect(getByText('Topic 1')).toBeDefined()
      expect(queryAllByTestId('space-link')).toHaveLength(5)
      expect(getAllByText('Space title')).toHaveLength(5)

      const images = getAllByRole('image')

      // expect 5 spaces with 4 images each (space cover + 3 user avatars)
      expect(images).toHaveLength(20)
      const [coverImage, avatarImage] = images
      expect(coverImage.props.source).toEqual({
        height: 315,
        uri: 'https://dev-images.holi.social/crop:0:0/resize:fill-down:944:630/ext:webp/quality:80/plain/https%3A%2F%2Fwww.example.com%2Fspace-cover.jpg',
        width: 472,
      })
      expect(avatarImage.props.source).toEqual({
        height: 24,
        uri: 'https://dev-images.holi.social/crop:0:0/resize:fill-down:48:48/ext:webp/quality:80/plain/https%3A%2F%2Fwww.example.com%2Fuser-avatar.jpg',
        width: 24,
      })

      expect(getByTestId('spaces-group-slider-see-more')).toBeDefined()

      fireEvent.press(getByTestId('spaces-group-slider-see-more'))

      expect(mockNavigate).toHaveBeenCalledWith('/spaces/filter?topic=topic-1')
    })
  })

  describe('with skill', () => {
    const renderSkill = (mocks: ReadonlyArray<MockedResponse>) =>
      render(
        <MockedProvider mocks={mocks} addTypename={false}>
          <SpacesGroupSlider skill={skill} pageSize={5} />
        </MockedProvider>
      )

    it('displays errors', async () => {
      const { getByText, getAllByTestId } = renderSkill([discoverSpacesErrorMock({ skillIds: ['skill1'] })])

      expect(getByText('Skill 1')).toBeDefined()

      expect(getAllByTestId('space-link-loading')).toHaveLength(5)

      await waitFor(() =>
        expect(global.mockOpenToast).toHaveBeenCalledWith('spaces.spaceListing.error', HoliToastType.ERROR, {
          testID: 'test-error',
        })
      )
    })

    it('renders correctly without see more', async () => {
      const { getByText, queryAllByTestId, getAllByText } = renderSkill([
        discoverSpacesDataMock({ skillIds: ['skill1'] }),
      ])

      expect(getByText('Skill 1')).toBeDefined()

      expect(queryAllByTestId('space-link-loading')).toHaveLength(5)

      await waitFor(async () => {
        expect(queryAllByTestId('space-link-loading')).toHaveLength(0)

        expect(getByText('Skill 1')).toBeDefined()
        expect(queryAllByTestId('space-link')).toHaveLength(5)
        expect(getAllByText('Space title')).toHaveLength(5)

        expect(queryAllByTestId('spaces-group-slider-see-more')).toHaveLength(0)
      })
    })
  })
})
