import React, { useEffect, useRef, useState } from 'react'
import { Animated, StyleSheet, TouchableOpacity } from 'react-native'

import { HoliIcon } from '@holi/icons/src/HoliIcon'
import { Check, Close } from '@holi/icons/src/generated'
import { useTheme } from '@holi/ui/styles/theme'

export const HoliSwitchSizes = ['small', 'medium', 'large'] as const
export type HoliSwitchSize = (typeof HoliSwitchSizes)[number]
export interface HoliSwitchProps {
  value: boolean
  onChange?: (value: boolean) => void
  size?: HoliSwitchSize
  disabled?: boolean
  duration?: number
  testID?: string
}

const HoliSwitch: React.FC<HoliSwitchProps> = ({
  value,
  onChange,
  size = 'medium',
  disabled = false,
  duration = 300,
  testID,
}) => {
  const { theme } = useTheme()
  const { colors } = theme
  const styles = getStyles()

  const circlePosXEnd = 44
  const circlePosXStart = 4
  const prevSwitchOnRef = useRef<boolean>()
  const prevSwitchOn = !!prevSwitchOnRef.current
  const [animXValue] = useState(new Animated.Value(value ? 1 : 0))

  useEffect(() => {
    const runAnimation = (): void => {
      const animValue = {
        fromValue: value ? 0 : 1,
        toValue: value ? 1 : 0,
        duration,
        useNativeDriver: false,
      }
      Animated.timing(animXValue, animValue).start()
    }
    prevSwitchOnRef.current = value
    if (prevSwitchOn !== value) runAnimation()
  }, [prevSwitchOn, value, animXValue, duration])

  const handleOnPress = (): void => {
    if (disabled) return
    onChange?.(!value)
  }

  let circleColor: string = colors.background10
  let backgroundColor: string = value ? colors.brandB300 : colors.gray300

  if (disabled) {
    circleColor = colors.background10
    backgroundColor = colors.foreground10
  }

  return (
    <TouchableOpacity testID={testID} onPress={handleOnPress} activeOpacity={0.8} disabled={disabled}>
      <Animated.View
        style={[
          styles.container,
          getSizeStyles(size),
          {
            borderWidth: 0,
            borderColor: 'transparent',
            backgroundColor: animXValue.interpolate({
              inputRange: [0, 1],
              outputRange: [backgroundColor, backgroundColor],
            }),
          },
        ]}
      >
        <Animated.View
          style={[
            styles.circle,
            getSizeStyles(size),
            {
              backgroundColor: circleColor,
            },
            {
              transform: [
                {
                  translateX: animXValue.interpolate({
                    inputRange: [0, 1],
                    outputRange: [circlePosXStart, circlePosXEnd] as string[] | number[],
                  }),
                },
              ],
            },
          ]}
        >
          {value ? (
            <HoliIcon icon={Check} color={colors.brandB300} size={26} />
          ) : (
            <HoliIcon icon={Close} color={colors.gray300} size={26} />
          )}
        </Animated.View>
      </Animated.View>
    </TouchableOpacity>
  )
}

const getSizeStyles = (size: HoliSwitchSize) => {
  switch (size) {
    case 'large':
      return { transform: [{ scaleX: 1 }, { scaleY: 1 }] }
    case 'small':
      return { transform: [{ scaleX: 0.5 }, { scaleY: 0.5 }] }
    default: // medium
      return { transform: [{ scaleX: 0.75 }, { scaleY: 0.75 }] }
  }
}

const getStyles = () =>
  StyleSheet.create({
    container: {
      flexDirection: 'row',
      alignItems: 'center',
      width: 80,
      height: 40,
      borderRadius: 100,
    },
    circle: {
      alignItems: 'center',
      justifyContent: 'center',
      borderRadius: 100,
      height: 32,
      width: 32,
    },
  })

export default HoliSwitch
